"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AcceptanceState = exports.AuthRequest = void 0;
const basic_auth_1 = __importDefault(require("basic-auth"));
const common_tags_1 = require("common-tags");
const handler_1 = require("./util/handler");
const session_1 = require("./session");
class AuthHandler extends handler_1.BaseHandler {
}
var AcceptanceState;
(function (AcceptanceState) {
    AcceptanceState[AcceptanceState["Pending"] = 0] = "Pending";
    AcceptanceState[AcceptanceState["Accepted"] = 1] = "Accepted";
    AcceptanceState[AcceptanceState["Rejected"] = 2] = "Rejected";
})(AcceptanceState || (exports.AcceptanceState = AcceptanceState = {}));
class AuthRequest extends handler_1.BaseRequest {
    constructor(context, clientId, protocol, req) {
        super(context);
        this.client = new session_1.Client(clientId);
        this.protocol = protocol;
        this.req = req;
        this._state = AcceptanceState.Pending;
    }
    accept() {
        if (this.state === AcceptanceState.Rejected) {
            this.context.logger.error((0, common_tags_1.oneLine) `accept() was called but auth request from client
        with id ${this.client.id} has already been rejected`);
            return;
        }
        this._state = AcceptanceState.Accepted;
    }
    reject(statusCode = 401) {
        if (this.state === AcceptanceState.Accepted) {
            this.context.logger.error((0, common_tags_1.oneLine) `reject() was called but auth request from client
        with id ${this.client.id} has already been accepted`);
            return;
        }
        this._state = AcceptanceState.Rejected;
        this._statusCode = statusCode;
    }
    get basicAuth() {
        return (0, basic_auth_1.default)(this.req);
    }
    get isEncrypted() {
        return this.req.socket.encrypted || false;
    }
    get isCertificateValid() {
        return this.req.socket.authorized || false;
    }
    get state() {
        return this._state;
    }
    get statusCode() {
        return this._statusCode;
    }
}
exports.AuthRequest = AuthRequest;
exports.default = AuthHandler;
//# sourceMappingURL=auth.js.map