"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const crypto_1 = require("crypto");
const eventemitter3_1 = __importDefault(require("eventemitter3"));
const common_tags_1 = require("common-tags");
const lodash_1 = __importDefault(require("lodash"));
const options_1 = require("./options");
const logger_1 = __importDefault(require("./util/logger"));
const handler_1 = require("./util/handler");
const Handlers = __importStar(require("./handlers"));
const session_1 = __importStar(require("./session"));
const type_1 = __importDefault(require("../types/ocpp/type"));
const call_1 = require("./call");
const callerror_1 = require("./callerror");
const auth_1 = require("./auth");
class BaseEndpoint extends eventemitter3_1.default {
    constructor(options, authHandlers = [], inboundHandlers = [], outboundHandlers = [], logger = logger_1.default, httpServer) {
        super();
        this.onHttpError = (err) => {
            this.logger.error('Error occured in HTTP(S) server');
            this.logger.trace(err.stack);
        };
        this.options = lodash_1.default.merge(options_1.defaultOptions, options);
        this.sessions = new Map();
        this.logger = logger;
        this.logger.debug('Loaded endpoint configuration');
        this.logger.trace(this.options);
        if (!httpServer) {
            if (this.options.certificateAuth) {
                httpServer = https_1.default.createServer({
                    ...this.options.tls,
                    requestCert: true,
                    rejectUnauthorized: false,
                });
            }
            else if (this.options.tls) {
                httpServer = https_1.default.createServer(this.options.tls);
            }
            else {
                httpServer = http_1.default.createServer();
            }
        }
        this.httpServer = httpServer;
        this.httpServer.on('error', this.onHttpError);
        if (!this.options.authRequired) {
            this.logger.warn((0, common_tags_1.oneLine) `options.authRequired is disabled,
        auth attempts will be accepted by default`);
        }
        if (this.options.certificateAuth && !(httpServer instanceof https_1.default.Server)) {
            this.logger.error((0, common_tags_1.oneLine) `options.certificateAuth is enabled
        but no https.Server instance was passed`);
        }
        else if (this.options.tls && !(httpServer instanceof https_1.default.Server)) {
            this.logger.error((0, common_tags_1.oneLine) `options.tls is set but no https.Server instance was passed`);
        }
        this.authHandlers = new handler_1.HandlerChain(new Handlers.PreconditionsHandler(), new Handlers.SessionTimeoutHandler(), ...authHandlers, new Handlers.DefaultAuthHandler());
        this.logger.debug(`Loaded ${this.authHandlers.size} auth handlers`);
        this.logger.trace(this.authHandlers.toString());
        this.inboundHandlers = new handler_1.HandlerChain(new Handlers.InboundMessageSynchronicityHandler(this.sessions, logger), new Handlers.InboundPendingMessageHandler(this.sessions), new Handlers.InboundActionsAllowedHandler(this.options, logger), ...inboundHandlers, new Handlers.DefaultMessageHandler(logger));
        this.logger.debug(`Loaded ${this.inboundHandlers.size} inbound handlers`);
        this.logger.trace(this.inboundHandlers.toString());
        this.outboundHandlers = new handler_1.HandlerChain(new Handlers.OutboundMessageSynchronicityHandler(this.sessions, logger), new Handlers.OutboundActionsAllowedHandler(this.options, logger), ...outboundHandlers, async (message) => await this.handleSend(message), new Handlers.OutboundPendingMessageHandler(this.sessions));
        this.logger.debug(`Loaded ${this.outboundHandlers.size} outbound handlers`);
        this.logger.trace(this.outboundHandlers.toString());
    }
    get isListening() {
        return this.httpServer.listening;
    }
    listen() {
        if (this.isListening) {
            this.logger.warn('listen() was called but endpoint is already listening for connections');
            this.logger.trace(new Error().stack);
            return;
        }
        this.logger.info('Starting endpoint', this.options);
        this.emit('server_starting', this.options);
        this.httpServer.listen(this.options.port, this.options.hostname, this.options.maxConnections, () => {
            this.logger.info(`Endpoint is listening on port ${this.options.port}`, this.options);
            this.emit('server_listening', this.options);
        });
    }
    stop() {
        if (!this.isListening) {
            this.logger.warn((0, common_tags_1.oneLine) `stop() was called but endpoint is
        currently not listening for connections`);
            this.logger.trace(new Error().stack);
            return;
        }
        this.logger.info('Stopping endpoint');
        this.emit('server_stopping');
        this.httpServer.close(err => {
            if (err) {
                this.logger.error('Error while stopping HTTP(S) server');
                this.logger.trace(err.stack);
            }
            else {
                this.logger.info('Stopped endpoint');
                this.emit('server_stopped');
            }
        });
    }
    auth(callback) {
        const handler = new Handlers.BasicAuthHandler(callback);
        this.authHandlers.add(handler, this.authHandlers.size - 2);
    }
    handle(action, callback) {
        const handler = new Handlers.ActionHandler(action, callback);
        this.inboundHandlers.add(handler, this.inboundHandlers.size - 2);
    }
    async send(recipient, action, data, id = (0, crypto_1.randomUUID)()) {
        const message = new call_1.OutboundCall(new session_1.Client(recipient), id, action, data);
        await this.onSend(message);
        return new Promise((resolve, reject) => {
            const callback = async (data) => {
                this.inboundHandlers.remove(handler);
                resolve(data);
            };
            const handler = new Handlers.IdHandler(id, callback);
            this.inboundHandlers.add(handler, this.inboundHandlers.size - 2);
        });
    }
    async onSend(message) {
        if (!this.isListening) {
            this.logger.warn((0, common_tags_1.oneLine) `onSend() was called but endpoint is
        currently not listening for connections`);
            this.logger.trace(new Error().stack);
            return;
        }
        if (!this.isConnected(message.recipient.id)) {
            this.logger.warn((0, common_tags_1.oneLine) `onSend() was called but client with
        id ${message.recipient.id} is not connected`);
            this.logger.trace(new Error().stack);
            return;
        }
        this.logger.debug((0, common_tags_1.oneLine) `Sending ${type_1.default[message.type]}
      to client with id ${message.recipient.id}`);
        this.logger.trace(message);
        await this.outboundHandlers.handle(message);
        this.logger.debug((0, common_tags_1.oneLine) `${type_1.default[message.type]} message to
      client with id ${message.recipient.id} was sent`);
        this.emit('message_sent', message);
    }
    onAuthRequest(request) {
        this.logger.debug(`Client with id ${request.client.id} attempting authentication`);
        this.emit('client_connecting', request);
        this.authHandlers.handle(request).then(result => {
            switch (result.state) {
                case auth_1.AcceptanceState.Rejected:
                    this.onAuthFailure(request);
                    break;
                case auth_1.AcceptanceState.Accepted:
                    this.onAuthSuccess(request);
                    break;
                case auth_1.AcceptanceState.Pending:
                default:
                    this.logger.warn((0, common_tags_1.oneLine) `Authentication request from client with
            id ${request.client.id} has not been handled`);
            }
        });
    }
    onAuthFailure(request) {
        const socket = request.req.socket;
        const status = request.statusCode;
        socket.write(`HTTP/1.1 ${status} ${http_1.default.STATUS_CODES[status]}\r\n\r\n`);
        socket.destroy();
        this.logger.warn((0, common_tags_1.oneLine) `Rejecting auth request from client with id ${request.client.id}
      with status: ${status} ${http_1.default.STATUS_CODES[status]}`);
        this.emit('client_rejected', request);
    }
    onAuthSuccess(request) {
        if (this.isConnected(request.client.id)) {
            this.logger.warn((0, common_tags_1.oneLine) `onAuthenticationSuccess() was called but client
        with id ${request.client.id} is already connected`);
            return;
        }
        const session = new session_1.default(request.client, request.protocol);
        this.sessions.set(request.client.id, session);
        this.emit('client_accepted', request);
        this.logger.info(`Client with id ${request.client.id} authenticated successfully`);
        this.emit('client_connected', session);
    }
    async onSessionClosed(clientId) {
        if (!this.sessions.delete(clientId)) {
            this.logger.error((0, common_tags_1.oneLine) `onSessionClosed() was called but session for client
        with id ${clientId} does not exist`);
            this.logger.trace(new Error().stack);
            return;
        }
        this.logger.info(`Client with id ${clientId} disconnected`);
        this.emit('client_disconnected', this.sessions.get(clientId));
    }
    async onInboundMessage(message) {
        this.logger.debug((0, common_tags_1.oneLine) `Received ${type_1.default[message.type]}
      message from client with id ${message.sender.id}`);
        this.logger.trace(message);
        this.emit('message_received', message);
        try {
            await this.inboundHandlers.handle(message);
        }
        catch (err) {
            if (err instanceof callerror_1.OutboundCallError) {
                await this.onSend(err);
            }
            else {
                this.logger.error(`Error occured while handling inbound
          ${type_1.default[message.type]} message`);
                this.logger.trace(err.stack);
            }
        }
    }
    get context() {
        return {
            endpoint: this,
            logger: this.logger,
        };
    }
}
exports.default = BaseEndpoint;
//# sourceMappingURL=endpoint.js.map