"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutboundActionsAllowedHandler = exports.InboundActionsAllowedHandler = void 0;
const common_tags_1 = require("common-tags");
const type_1 = __importDefault(require("../../types/ocpp/type"));
const call_1 = require("../call");
const callerror_1 = require("../callerror");
const handler_1 = require("../handler");
class InboundActionsAllowedHandler extends handler_1.InboundMessageHandler {
    constructor(config, logger) {
        super();
        this.config = config;
        this.logger = logger;
    }
    async handle(message) {
        if (message instanceof call_1.InboundCall &&
            !this.config.actionsAllowed.includes(message.action)) {
            this.logger.warn((0, common_tags_1.oneLine) `Received ${type_1.default[message.type]}
        message with unsupported action: ${message.action}`);
            throw new callerror_1.OutboundCallError(message.sender, message.id, 'NotImplemented', `Action '${message.action}' is not supported`);
        }
        return await super.handle(message);
    }
}
exports.InboundActionsAllowedHandler = InboundActionsAllowedHandler;
class OutboundActionsAllowedHandler extends handler_1.OutboundMessageHandler {
    constructor(config, logger) {
        super();
        this.config = config;
        this.logger = logger;
    }
    async handle(message) {
        if (message instanceof call_1.OutboundCall &&
            !this.config.actionsAllowed.includes(message.action)) {
            this.logger.warn((0, common_tags_1.oneLine) `Attempting to send ${type_1.default[message.type]}
        message with unsupported action: ${message.action}`);
            return;
        }
        return await super.handle(message);
    }
}
exports.OutboundActionsAllowedHandler = OutboundActionsAllowedHandler;
//# sourceMappingURL=message-actions.js.map