import { Client } from './session';
import MessageType from '../types/ocpp/type';
import { InboundMessageHandler, ResponseHandler } from './handler';
import { RequestContext } from './util/handler';
type Payload = any;
declare abstract class OcppMessage {
    context: RequestContext;
    readonly type: MessageType;
    readonly id: string;
    protected _timestamp?: Date;
    constructor(id: string);
    get timestamp(): Date;
}
declare abstract class InboundMessage extends OcppMessage {
    readonly sender: Client;
    constructor(sender: Client, id: string);
}
declare abstract class OutboundMessage extends OcppMessage {
    recipient: Client;
    private _isSent;
    constructor(recipient: Client, id: string);
    setSent(): void;
    get isSent(): boolean;
}
declare abstract class RespondableMessage<TResponse extends OutboundMessage> extends InboundMessage {
    private _responseHandler?;
    private _response?;
    constructor(sender: Client, id: string, responseHandler?: ResponseHandler<TResponse>);
    respond(response: TResponse): Promise<void>;
    set responseHandler(handler: ResponseHandler<TResponse>);
    get isResponded(): boolean;
    get response(): OutboundMessage;
}
declare abstract class ResultingMessage<TResponse extends InboundMessage> extends OutboundMessage {
    private _responseHandler?;
    private _response?;
    constructor(recipient: Client, id: string, responseHandler?: InboundMessageHandler<TResponse>);
    onResponse(response: TResponse): Promise<void>;
    set responseHandler(handler: InboundMessageHandler<TResponse>);
    get hasResponse(): boolean;
    get response(): TResponse;
}
export default OcppMessage;
export { Payload, InboundMessage, OutboundMessage, RespondableMessage, ResultingMessage, };
