"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResultingMessage = exports.RespondableMessage = exports.OutboundMessage = exports.InboundMessage = void 0;
class OcppMessage {
    constructor(id) {
        this.id = id;
        this._timestamp = null;
    }
    get timestamp() {
        return this._timestamp;
    }
}
class InboundMessage extends OcppMessage {
    constructor(sender, id) {
        super(id);
        this.sender = sender;
        this._timestamp = new Date();
    }
}
exports.InboundMessage = InboundMessage;
class OutboundMessage extends OcppMessage {
    constructor(recipient, id) {
        super(id);
        this.recipient = recipient;
        this._isSent = false;
    }
    setSent() {
        this._isSent = true;
        this._timestamp = new Date();
    }
    get isSent() {
        return this._isSent;
    }
}
exports.OutboundMessage = OutboundMessage;
class RespondableMessage extends InboundMessage {
    constructor(sender, id, responseHandler) {
        super(sender, id);
        this._responseHandler = responseHandler || null;
        this._response = null;
    }
    async respond(response) {
        if (!this._responseHandler) {
            throw new Error('respond() was called but responseHandler is not set');
        }
        await this._responseHandler(response);
        this._response = response;
    }
    set responseHandler(handler) {
        this._responseHandler = handler;
    }
    get isResponded() {
        return !!this._response;
    }
    get response() {
        return this._response;
    }
}
exports.RespondableMessage = RespondableMessage;
class ResultingMessage extends OutboundMessage {
    constructor(recipient, id, responseHandler) {
        super(recipient, id);
        this._responseHandler = responseHandler || null;
    }
    async onResponse(response) {
        if (!this._responseHandler) {
            throw new Error('onResponse() was called but responseHandler is not set');
        }
        this._response = response;
        await this._responseHandler.handle(response);
    }
    set responseHandler(handler) {
        this._responseHandler = handler;
    }
    get hasResponse() {
        return !!this._response;
    }
    get response() {
        return this._response;
    }
}
exports.ResultingMessage = ResultingMessage;
exports.default = OcppMessage;
//# sourceMappingURL=message.js.map