import { Logger } from 'ts-log';
import { Endpoint } from '../endpoint';
interface Handler<TRequest extends Request> {
    handle: HandlerFunction<TRequest>;
    set next(handler: Handler<TRequest>);
}
type HandlerFunction<TRequest> = (request: TRequest) => Promise<TRequest>;
type HandlerRequest<THandler> = THandler extends Handler<infer TRequest> ? TRequest : never;
interface Request {
    context: RequestContext;
}
type RequestContext = {
    endpoint: Endpoint;
    logger: Logger;
};
declare abstract class BaseHandler<TRequest extends Request> implements Handler<TRequest> {
    private _next;
    set next(handler: BaseHandler<TRequest>);
    handle(request: TRequest): Promise<TRequest>;
    static fromFunction<TRequest extends Request>(handler: HandlerFunction<TRequest>): Handler<TRequest>;
}
declare class BaseRequest implements Request {
    readonly context: RequestContext;
    constructor(context: RequestContext);
}
declare class HandlerChain<THandler extends Handler<TRequest>, TRequest extends Request = HandlerRequest<THandler>> {
    private handlers;
    constructor(...handlers: Array<THandler | HandlerFunction<TRequest>>);
    handle(request: HandlerRequest<THandler>): Promise<TRequest>;
    add(handler: THandler | HandlerFunction<TRequest>, pos?: number): void;
    remove(handler: THandler): void;
    private mapHandlers;
    get size(): number;
    toString(): string;
}
export default Handler;
export { BaseHandler, BaseRequest, Request, RequestContext, HandlerChain, HandlerFunction, HandlerRequest, };
