"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HandlerChain = exports.BaseRequest = exports.BaseHandler = void 0;
const logger_1 = require("./logger");
class BaseHandler {
    set next(handler) {
        this._next = handler;
    }
    async handle(request) {
        if (this._next) {
            return await this._next.handle(request);
        }
        else {
            return request;
        }
    }
    static fromFunction(handler) {
        return new (class AnonymousHandler extends BaseHandler {
            async handle(request) {
                return await super.handle(await handler(request));
            }
        })();
    }
}
exports.BaseHandler = BaseHandler;
class BaseRequest {
    constructor(context) {
        this.context = context;
    }
}
exports.BaseRequest = BaseRequest;
class HandlerChain {
    constructor(...handlers) {
        this.handlers = handlers.map(handler => typeof handler === 'function'
            ? BaseHandler.fromFunction(handler)
            : handler);
        this.mapHandlers();
    }
    async handle(request) {
        return await this.handlers[0].handle(request);
    }
    add(handler, pos = this.size) {
        if (typeof handler === 'function') {
            handler = BaseHandler.fromFunction(handler);
        }
        this.handlers.splice(pos, 0, handler);
        this.mapHandlers();
    }
    remove(handler) {
        this.handlers = this.handlers.filter(_handler => _handler !== handler);
        this.mapHandlers();
    }
    mapHandlers() {
        this.handlers.forEach((handler, i, handlers) => {
            handler.next = handlers[i + 1];
        });
    }
    get size() {
        return this.handlers.length;
    }
    toString() {
        return (0, logger_1.logObject)({
            ...this.handlers.map(handler => handler.constructor.name),
        });
    }
}
exports.HandlerChain = HandlerChain;
//# sourceMappingURL=handler.js.map