/// <reference types="node" />
/// <reference types="node" />
import http from 'http';
import * as ws from 'ws';
import { Duplex } from 'stream';
import { Logger } from 'ts-log';
import BaseEndpoint from '../common/endpoint';
import EndpointOptions from '../common/options';
import { Client } from '../common/session';
import { OutboundMessage } from '../common/message';
import AuthHandler from '../common/auth';
import { InboundMessageHandler, OutboundMessageHandler } from '../common/handler';
import WsValidator from './ws-validator';
declare class WsEndpoint extends BaseEndpoint {
    readonly sockets: Map<string, ws.WebSocket>;
    protected wsServer: ws.Server;
    protected validator: WsValidator;
    constructor(options?: EndpointOptions, authHandlers?: AuthHandler[], inboundHandlers?: InboundMessageHandler[], outboundHandlers?: OutboundMessageHandler[], logger?: Logger, httpServer?: http.Server, validator?: WsValidator);
    isConnected(clientId: string): boolean;
    drop(clientId: string, force?: boolean, code?: number, data?: string | Buffer): Promise<void>;
    protected handleSend: (message: OutboundMessage) => Promise<OutboundMessage>;
    protected onHttpUpgrade: (req: http.IncomingMessage, socket: Duplex, head: Buffer) => void;
    protected onWsConnected: (ws: ws.WebSocket, req: http.IncomingMessage, client: Client) => void;
    protected onWsDisconnected: (clientId: string, code: number, reason: Buffer) => void;
    protected parseMessage(rawMessage: string): {
        type: number;
        id: string;
        action: "Authorize" | "BootNotification" | "ChangeAvailability" | "ChangeConfiguration" | "ClearCache" | "GetConfiguration" | "Heartbeat" | "MeterValues" | "RemoteStartTransaction" | "RemoteStopTransaction" | "Reset" | "StartTransaction" | "StatusNotification" | "StopTransaction" | "UnlockConnector";
        payload: any;
        errorCode: any;
        errorDescription: any;
        errorDetails: any;
    };
}
export default WsEndpoint;
