"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ws = __importStar(require("ws"));
const path_1 = __importDefault(require("path"));
const crypto_1 = require("crypto");
const common_tags_1 = require("common-tags");
const endpoint_1 = __importDefault(require("../common/endpoint"));
const call_1 = require("../common/call");
const callresult_1 = require("../common/callresult");
const callerror_1 = require("../common/callerror");
const type_1 = __importDefault(require("../types/ocpp/type"));
const auth_1 = require("../common/auth");
const ws_validator_1 = __importDefault(require("./ws-validator"));
class WsEndpoint extends endpoint_1.default {
    constructor(options, authHandlers, inboundHandlers, outboundHandlers, logger, httpServer, validator = new ws_validator_1.default()) {
        super(options, authHandlers, inboundHandlers, outboundHandlers, logger, httpServer);
        this.handleSend = async (message) => {
            const ws = this.sockets.get(message.recipient.id);
            const session = this.sessions.get(message.recipient.id);
            const messageArr = [message.type, message.id];
            if (message instanceof call_1.OutboundCall) {
                messageArr.push(message.action, message.data);
            }
            else if (message instanceof callresult_1.OutboundCallResult) {
                messageArr.push(message.data);
            }
            else if (message instanceof callerror_1.OutboundCallError) {
                messageArr.push(message.code, message.description, message.details);
            }
            if (this.options.validation &&
                (message instanceof call_1.OutboundCall || message instanceof callresult_1.OutboundCallResult)) {
                const rawData = JSON.parse(JSON.stringify(message.data), (k, v) => v);
                const messageValidation = await this.validator.validate(message.type, message.action || session.pendingInboundMessage.action, rawData, ws.protocol);
                if (!(messageValidation === null || messageValidation === void 0 ? void 0 : messageValidation.valid)) {
                    return;
                }
            }
            return new Promise((resolve, reject) => {
                ws.send(JSON.stringify(messageArr), err => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(message);
                    }
                });
            });
        };
        this.onHttpUpgrade = (req, socket, head) => {
            var _a;
            const clientId = path_1.default.basename(req.url);
            const clientProtocols = (_a = req.headers['sec-websocket-protocol']) === null || _a === void 0 ? void 0 : _a.split(',');
            const supportedProtocols = this.options.protocols.filter(protocol => clientProtocols === null || clientProtocols === void 0 ? void 0 : clientProtocols.includes(protocol));
            const authRequest = new auth_1.AuthRequest(this.context, clientId, supportedProtocols[0], req);
            if (!clientProtocols) {
                this.logger.warn((0, common_tags_1.oneLine) `Client with id ${clientId} attempted authentication
        without specifying any WebSocket subprotocol`);
                authRequest.reject(400);
                this.onAuthFailure(authRequest);
                return;
            }
            if (supportedProtocols.length === 0) {
                this.logger.warn((0, common_tags_1.oneLineInlineLists) `Client with id ${clientId}
        attempted authentication with unsupported
        subprotocol(s): ${clientProtocols}`);
                authRequest.reject(400);
                this.onAuthFailure(authRequest);
                return;
            }
            const listener = (request) => {
                if (request.client.id === clientId) {
                    this.logger.debug((0, common_tags_1.oneLine) `Upgrading WebSocket connection
          with protocol ${supportedProtocols[0]}`);
                    this.wsServer.handleUpgrade(req, socket, head, ws => {
                        ws.on('close', (code, reason) => this.onWsDisconnected(clientId, code, reason));
                        this.sockets.set(clientId, ws);
                        this.wsServer.emit('connection', ws, req, authRequest.client);
                    });
                    this.removeListener('client_accepted', listener);
                }
            };
            this.on('client_accepted', listener);
            this.onAuthRequest(authRequest);
        };
        this.onWsConnected = (ws, req, client) => {
            ws.on('message', async (data, isBinary) => {
                if (isBinary) {
                    this.logger.warn(`Received message with binary data from client with
          id ${client.id} which is currently not supported`);
                    this.drop(client.id);
                    return;
                }
                let messageProperties;
                try {
                    messageProperties = this.parseMessage(data.toString());
                }
                catch (err) {
                    this.logger.warn(`Error while parsing message from client
          with id ${client.id}: ${err.message}`);
                    this.logger.trace(err.stack);
                    const errorResponse = new callerror_1.OutboundCallError(client, (0, crypto_1.randomBytes)(16).toString('hex'), 'ProtocolError', `Failed to parse message: ${err.message}`, null);
                    await this.onSend(errorResponse);
                    return;
                }
                const { type, id, action, payload, errorCode, errorDescription, errorDetails, } = messageProperties;
                if (this.options.validation &&
                    (type === type_1.default.CALL || type === type_1.default.CALLRESULT)) {
                    const session = this.sessions.get(client.id);
                    const lastOutboundCall = session === null || session === void 0 ? void 0 : session.lastOutboundMessage;
                    const messageValidation = await this.validator.validate(type, action || lastOutboundCall.action, payload, ws.protocol);
                    if (!(messageValidation === null || messageValidation === void 0 ? void 0 : messageValidation.valid)) {
                        return;
                    }
                }
                const responseHandler = async (response) => {
                    await this.onSend(response);
                };
                let message;
                switch (type) {
                    case type_1.default.CALL:
                        message = new call_1.InboundCall(client, id, action, payload, responseHandler);
                        break;
                    case type_1.default.CALLRESULT:
                        message = new callresult_1.InboundCallResult(client, id, payload);
                        break;
                    case type_1.default.CALLERROR:
                        message = new callerror_1.InboundCallError(client, id, errorCode, errorDescription, errorDetails);
                        break;
                }
                this.onInboundMessage(message);
            });
        };
        this.onWsDisconnected = (clientId, code, reason) => {
            this.logger.debug(`WebSocket connection closed with reason: ${code} ${reason}`);
            this.onSessionClosed(clientId);
        };
        this.sockets = new Map();
        this.wsServer = new ws.Server({ noServer: true });
        this.validator = validator;
        this.httpServer.on('upgrade', this.onHttpUpgrade);
        this.wsServer.on('connection', this.onWsConnected);
        this.wsServer.shouldHandle = req => req.url.startsWith(this.options.route);
    }
    isConnected(clientId) {
        var _a;
        return ((_a = this.sockets.get(clientId)) === null || _a === void 0 ? void 0 : _a.readyState) === ws.WebSocket.OPEN;
    }
    async drop(clientId, force = false, code = 1000, data) {
        const ws = this.sockets.get(clientId);
        if (!ws) {
            this.logger.warn(`drop() was called with invalid client id ${clientId}`);
            return;
        }
        if (force) {
            ws.terminate();
        }
        else {
            ws.close(code, data);
        }
    }
    parseMessage(rawMessage) {
        let message;
        try {
            message = JSON.parse(rawMessage);
        }
        catch (err) {
            throw new Error('Invalid JSON format', { cause: err });
        }
        if (!Array.isArray(message)) {
            throw new Error('Message is not an array');
        }
        const type = message[0];
        if (typeof type !== 'number' ||
            !Object.values(type_1.default).includes(type)) {
            throw new Error('Missing or invalid type field');
        }
        const isCallMessage = type === type_1.default.CALL;
        const isCallResultMessage = type === type_1.default.CALLRESULT;
        const isCallErrorMessage = type === type_1.default.CALLERROR;
        const id = message[1];
        if (typeof id !== 'string') {
            throw new Error('Missing or invalid id field');
        }
        const action = isCallMessage ? message[2] : null;
        if (isCallMessage && typeof action !== 'string') {
            throw new Error('Missing or invalid action field');
        }
        const errorCode = isCallErrorMessage ? message[2] : null;
        const errorDescription = isCallErrorMessage ? message[3] : null;
        const errorDetails = isCallErrorMessage ? message[4] : null;
        if (isCallErrorMessage &&
            (typeof errorCode !== 'string' || typeof errorDescription !== 'string')) {
            throw new Error('Missing or invalid error code or description field');
        }
        const payload = isCallMessage
            ? message[3]
            : isCallResultMessage
                ? message[2]
                : null;
        return {
            type,
            id,
            action,
            payload,
            errorCode,
            errorDescription,
            errorDetails,
        };
    }
}
exports.default = WsEndpoint;
//# sourceMappingURL=ws-endpoint.js.map